/*
 * GPIO_lib.h
 *
 *  Created on: 2011/11/1
 *      Author: Eric.Lai
 */

#include "HY16F188.h"

#ifndef GPIO_LIB_H_
#define GPIO_LIB_H_

#define PT1_BASE (GPIO_BASE+OFFSET00)
#define PT2_BASE (GPIO_BASE+OFFSET10)
#define PT3_BASE (GPIO_BASE+OFFSET20)
#define PT4_BASE (GPIO_BASE+OFFSET30)
#define OUTPUT_MODE_OFFICE OFFSET00
#define GOIP_CLOCK_BASE (CLOCK_BASE+OFFSET0C)

#ifdef  INTERRUPT_BASE
#define GPIO_INTERRUPT_CTRL  (INTERRUPT_BASE+OFFSET10)
#endif

#ifndef E_SUCCESS
#define E_SUCCESS		0
#endif

#ifndef E_FAIL
#define E_FAIL		1
#endif
#define E_DRVGPIO_ARGUMENT E_FAIL
#define E_DRVGPIO_ARGUMENT2 0xf0
#define E_DRVGPIO_ARGUMENT3 0xff000000

#ifdef GPIO_BASE
#define GPIO_INPUT_SET   (GPIO_BASE+OFFSET04)
#define GPIO_OP_CTRL     (GPIO_BASE+OFFSET40)
#define GPIO_PT3_PULLHIGH  (GPIO_BASE+OFFSET20)
#define GPIO_PT3_OUTPUT  (GPIO_BASE+OFFSET20)
#define GPIO_PT3_INPUT  (GPIO_BASE+OFFSET24)
#endif

//# OPAMP Bit mapping
#define  OPPTE         18
#define  OPPTS         19
#define  OPPTE_MASK    (OPPTE+MASK)
#define  OPPTS_MASK    (OPPTS+MASK)
#define  OPPTS_MAX     1
#define  OPO_BIT       7
#define  OPO1_BIT      0

#define  PORT_OFFSET   0x10
#define  INT_PORT_OFFSET   0x4

typedef enum
{
	E_PT0 = 0,
	E_PT1 = 1,
	E_PT2 = 2,
	E_PT3 = 3,
	E_PT4 = 4
} E_DRVGPIO_PORT;

typedef enum
{
    E_IO_INPUT = 0,
    E_IO_OUTPUT,
    E_IO_PullHigh,
    E_IO_IntEnable
} E_DRVGPIO_IO;

typedef enum
{
	E_DisableGPIOInt = 0,
	E_P_Edge,
	E_N_Edge,
	E_Chang_Level,
	E_LLTri,
	E_LHTri,
	E_LLTri1,
	E_LHTri1
} E_DRVGPIO_IntTriMethod;

typedef enum
{
    E_HS_CK= 0,
    E_LS_CK=1
} E_DRVGPIO_CLK_SOURCE;

unsigned int DrvGPIO_Open (uint32_t port,uint32_t i32Bit,uint32_t mode );
unsigned int DrvGPIO_Close(uint32_t port,uint32_t i32Bit,uint32_t mode );
unsigned int DrvGPIO_IntTrigger ( uint32_t port, uint32_t i32Bit, uint32_t mode);
unsigned int DrvGPIO_ClkGenerator ( uint32_t uClk, uint32_t uDivider);
unsigned int DrvGPIO_GetPortBits (unsigned int port);
unsigned int DrvGPIO_SetPortBits (unsigned int uport, unsigned int ui32Data);
unsigned int DrvGPIO_GetBit (unsigned int port, unsigned int ui8Bit);
unsigned int DrvGPIO_SetBit (unsigned int uport, unsigned int i32Bit);
unsigned int DrvGPIO_ClrBit (unsigned int uport, unsigned int i32Bit);
unsigned int DrvGPIO_ClrPortBits (unsigned int uport, unsigned int ui32Data);
unsigned int DrvGPIO_ClearIntFlag(E_DRVGPIO_PORT port,unsigned int uint32);
uint32_t DrvGPIO_GetIntFlag(E_DRVGPIO_PORT port);
unsigned char DrvGPIO_EnableAnalogPin(short int port,unsigned int i32Bit);


//PT1 ADD SIMPLE FUNCTION FOR reduce the code size
void DrvGPIO_PT1_EnableINPUT(short int ubit);
void DrvGPIO_PT1_DisableINPUT(short int ubit);
void DrvGPIO_PT1_EnablePullHigh(short int ubit);
void DrvGPIO_PT1_DisablePullHigh(short int ubit);
void DrvGPIO_PT1_EnableOUTPUT(short int ubit);
void DrvGPIO_PT1_DisableOUTPUT(short int ubit);
void DrvGPIO_PT1_EnableINT(short int ubit);
void DrvGPIO_PT1_DisableINT(short int ubit);
void DrvGPIO_PT1_IntTriggerPorts(uint32_t i32Bit, uint32_t mode);
void DrvGPIO_PT1_IntTriggerBit(uint32_t i32Bit, uint32_t mode);
void DrvGPIO_PT1_ClearIntFlag(short int uint32);
unsigned char DrvGPIO_PT1_GetIntFlag(void);
unsigned char DrvGPIO_PT1_GetPortBits (void);
void DrvGPIO_PT1_SetPortBits (unsigned char ui32Data);
void DrvGPIO_PT1_ClrPortBits (unsigned int ui32Data);

//PT2 ADD SIMPLE FUNCTION FOR reduce the code size
void DrvGPIO_PT2_EnableINPUT(short int ubit);
void DrvGPIO_PT2_DisableINPUT(short int ubit);
void DrvGPIO_PT2_EnablePullHigh(short int ubit);
void DrvGPIO_PT2_DisablePullHigh(short int ubit);
void DrvGPIO_PT2_EnableOUTPUT(short int ubit);
void DrvGPIO_PT2_DisableOUTPUT(short int ubit);
void DrvGPIO_PT2_EnableINT(short int ubit);
void DrvGPIO_PT2_DisableINT(short int ubit);
void DrvGPIO_PT2_IntTriggerPorts(uint32_t i32Bit, uint32_t mode);
void DrvGPIO_PT2_IntTriggerBit(uint32_t i32Bit, uint32_t mode);
void DrvGPIO_PT2_ClearIntFlag(short int uint32);
unsigned char DrvGPIO_PT2_GetIntFlag(void);
unsigned char DrvGPIO_PT2_GetPortBits (void);
void DrvGPIO_PT2_SetPortBits (unsigned char ui32Data);
void DrvGPIO_PT2_ClrPortBits (unsigned int ui32Data);


//PT3 ADD SIMPLE FUNCTION FOR reduce the code size
void DrvGPIO_PT3_EnableINPUT(short int ubit);
void DrvGPIO_PT3_DisableINPUT(short int ubit);
void DrvGPIO_PT3_EnablePullHigh(short int ubit);
void DrvGPIO_PT3_DisablePullHigh(short int ubit);
void DrvGPIO_PT3_EnableOUTPUT(short int ubit);
void DrvGPIO_PT3_DisableOUTPUT(short int ubit);
unsigned char DrvGPIO_PT3_GetPortBits (void);
void DrvGPIO_PT3_SetPortBits (unsigned char ui32Data);
void DrvGPIO_PT3_ClrPortBits (unsigned int ui32Data);

#endif /* GPIO_LIB_H_ */
