/*
 * DrvUART.c
 *
 *  Created on: 2011/11/1
 *      Author: Eric.Lai
 */
#include "DrvUART.h"
#include "SpecialMacro.h"

#include "stdio.h"
#include "ctype.h"
#include "HY16F188.h"
#include "hyprintf.h"
/****************************************************************************
*name:DrvUART_Open()
*function: setting the CLOCK /BaudRate/DataBits/outputPin for UART
*input: no
*output: 
*version: v1.1
*date:  2013-11-29
*modify:    delect the code about enbable the UART clock source 
*           change 'uclock' data type from 'unsigned int ' to 'float'; 20160111
****************************************************************************/
unsigned int DrvUART_Open(
		       float uClock,
		unsigned int uBaudRate ,
		unsigned int uParity,
		unsigned int uDataBits,
		unsigned int uOuputPin)
{
	unsigned int l_reg,l_data,ReturnVal,l_temp;
	ReturnVal=0;
	if(uClock>20){return E_UART_ERR_CLOCK;}
	if(uBaudRate==0){return E_UART_ERR_BAUDRAET;}
	else{
		l_reg=R_UART_CLOCK;
		l_temp=((inw(l_reg)&(UCLK_Div_MAX<<UCLK_Div))>>UCLK_Div);
		l_data=((uClock*1000000/(1<<l_temp)/uBaudRate/4)-1)+0.5;
		l_reg=R_UART_BRGRH;
		outw(l_reg,(l_data>>8));
		l_reg=R_UART_BRGRL;
		outw(l_reg,(l_data&0x000000ff));
	}

	if(uParity>2){return E_UART_ERR_PARITY;}
	else{
		switch(uParity)
		{
		    case 0:
		    	l_temp=(0<<ENADD)|(1<<ENADD_MASK);
		    	l_data=(0<<PARITY)|(1<<PARITY_MASK);
		    break;
		    case 1:
		    	l_temp=(1<<ENADD)|(1<<ENADD_MASK);
		    	l_data=(0<<PARITY)|(1<<PARITY_MASK);
		    break;
		    case 2:
		    	l_temp=(1<<ENADD)|(1<<ENADD_MASK);
		    	l_data=(1<<PARITY)|(1<<PARITY_MASK);
		    break;
		    default: break;
		}
		l_reg=R_UART_CTRL2;
		outw(l_reg,l_temp);
		l_reg=R_UART_CTRL1;
		outw(l_reg,l_data);
	}
	if(uDataBits>1){return E_UART_ERR_DATABIT;}
	else{
		if(uDataBits==0){l_data=(0<<TX9)|(1<<TX9_MASK);
		                 l_temp=(0<<RC9)|(1<<RC9_MASK);
		}
		else{l_data=(1<<TX9)|(1<<TX9_MASK);
		     l_temp=(1<<RC9)|(1<<RC9_MASK);
		}
		l_reg=R_UART_CTRL1;
		outw(l_reg,l_data);
		l_reg=R_UART_CTRL2;
		outw(l_reg,l_temp);
	}
	if(uOuputPin>UAPS_MAX){return E_UART_ERR_OUTPIN;}
	else{l_data=(uOuputPin<<UAPS)|(UAPS_MAX<<UAPS_MASK)|(1<<UAPTOE)|(1<<UAPTOE_MASK);
	   l_reg=R_UART_GPIO;
	   outw(l_reg,l_data);
	}
	l_data=(1<<ENSP)|(1<<ENSP_MASK)|(1<<ENTX)|(1<<ENTX_MASK);
	l_reg=R_UART_CTRL1;
	outw(l_reg,l_data);
	l_data=(1<<ENCR)|(1<<ENCR_MASK);
	l_reg=R_UART_CTRL2;
	outw(l_reg,l_data);
	return ReturnVal;
}

void DrvUART_Close(void)
{
	unsigned int l_reg,l_data;
	l_data=(1<<ENTX_MASK)|(1<<ENSP_MASK);
	l_reg=R_UART_CTRL1;
	outw(l_reg,l_data);
}
/****************************************************************************
*name    : DrvUART_EnableInt(unsigned int uTXIE, unsigned int uRXIE)
*function: Enable the RX AND TX interrupt
*input   : uTXIE/uRXIE
*          0 disable the interrupt
*          1 enable the interrupt
*output  : 0 success
*          others: fail
*version : v1.1
*date    : 2013-11-28
*modify  : change the UART_TXIE/UART_RXIE value;
****************************************************************************/
unsigned int DrvUART_EnableInt(unsigned int uTXIE, unsigned int uRXIE)
{
	unsigned int l_reg,l_data,ReturnVal;
	ReturnVal=0;
	l_reg=R_UART_INT_CTRL;
	if(uTXIE>1){return E_DRVUART_ARGUMENT;}
	else if(uTXIE==0){l_data=(0<<UART_TXIE)|(1<<UART_TXIE_MASK);}
	else if(uTXIE==1){l_data=(1<<UART_TXIE)|(1<<UART_TXIE_MASK);}
	if(uRXIE>1){return E_DRVUART_ARGUMENT;}
	else if(uRXIE==0){l_data|=(0<<UART_RXIE)|(1<<UART_RXIE_MASK);}
	else if(uRXIE==1){l_data|=(1<<UART_RXIE)|(1<<UART_RXIE_MASK);}
	outw(l_reg,l_data);
	if((uTXIE==1)|(uRXIE==1)){
	    //l_data=__nds32__mfsr(NDS32_SR_INT_MASK);
	    //SETBIT(l_data,0);
	    //__nds32__mtsr(l_data, NDS32_SR_INT_MASK);
	}
	else{
		//l_data=__nds32__mfsr(NDS32_SR_INT_MASK);
		//CLRBIT(l_data,0);
		//__nds32__mtsr(l_data, NDS32_SR_INT_MASK);
	}
	return ReturnVal;
}
/****************************************************************************
*name:DrvUART_GetTxFlag()
*function: read the TXIF
*input: no
*output: TXIF value
*version: v1.1
*date:  2013-11-28
*modify:change the UART_TXIF value from 2 to 3;
****************************************************************************/
unsigned int DrvUART_GetTxFlag()
{
	unsigned int l_reg,ReturnVal;
	ReturnVal=0;
	l_reg=R_UART_INT_CTRL;
	ReturnVal=((inw(l_reg)&(1<<UART_TXIF))>>UART_TXIF);
	return ReturnVal;
}
/****************************************************************************
*name:DrvUART_GetRxFlag()
*function: read the RXIF
*input: no
*output: RXIF value
*version: v1.1
*date:  2013-11-28
*modify:Change the UART_RXIF value from 1 to 2;
****************************************************************************/
unsigned int DrvUART_GetRxFlag(void)
{
	unsigned int l_reg,ReturnVal;
	ReturnVal=0;
	l_reg=R_UART_INT_CTRL;
	ReturnVal=((inw(l_reg)&(1<<UART_RXIF))>>UART_RXIF);
	return ReturnVal;
}

void DrvUART_ClrTxFlag()
{
	unsigned int l_reg,l_data;
	l_reg=R_UART_INT_CTRL;
	l_data=(1<<UART_TXIF_MASK);
	outw(l_reg,l_data);
}

void DrvUART_ClrRxFlag()
{
	unsigned int l_reg,l_data;
	l_reg=R_UART_INT_CTRL;
	l_data=(1<<UART_RXIF_MASK);
	outw(l_reg,l_data);
}

unsigned int DrvUART_Read()
{
	unsigned int l_reg;
	l_reg=R_UART_RCREG;
	return (inw(l_reg)&0x000000ff);
}

unsigned int DrvUART_Read9Bit()
{
	unsigned int l_reg,l_data;
	l_reg=R_UART_RCREG;
	l_data=(inw(l_reg)&0x000000ff);
	l_reg=R_UART_FLAG;
	l_data|=(((inw(l_reg)>>RC9D)&0x00000001)<<8);
	return l_data;
}

void DrvUART_Write(unsigned int uData)
{
	unsigned int l_reg,l_data;
	l_reg=R_UART_TXREG;
	l_data=uData;
	outw(l_reg,l_data);
}

void DrvUART_EnableWakeUp()
{
	unsigned int l_reg,l_data;
	l_reg=R_UART_CTRL1;
	l_data=(1<<WUE)|(1<<(WUE_MASK));
	outw(l_reg,l_data);
}

/*void DrvUART_DisableWakeUp()
{
	unsigned int l_reg,l_data;
	l_reg=R_UART_CTRL1;
	l_data=(1<<WUE_MASK);
	outw(l_reg,l_data);
}*/

void DrvUART_Enable_AutoBaudrate()
{
	unsigned int l_reg,l_data;
	l_reg=R_UART_CTRL2;
	l_data=(1<<ENABD)|(1<<(ENABD_MASK));
	outw(l_reg,l_data);
}

void DrvUART_DisableAutoBaudrate()
{
	unsigned int l_reg,l_data;
	l_reg=R_UART_CTRL2;
	l_data=(1<<ENABD_MASK);
	outw(l_reg,l_data);
}

/*
modify: change '&PERR' to' &0x20'   20150922
*/
unsigned int DrvUART_GetPERR()
{
	unsigned int l_reg,ReturnVal;
	ReturnVal=0;
	l_reg=R_UART_FLAG;
	ReturnVal=((inw(l_reg)&0x20)>>PERR);
	return ReturnVal;
}
/*
modify: change '&FERR' to' &0x10'   20150922
*/
unsigned int DrvUART_GetFERR()
{
	unsigned int l_reg,ReturnVal;
	ReturnVal=0;
	l_reg=R_UART_FLAG;
	ReturnVal=((inw(l_reg)&0x10)>>FERR);
	return ReturnVal;
}

void DrvUART_ClrFERR()
{
	unsigned int l_reg,l_data;
	l_reg=R_UART_CTRL2;
	l_data=(1<<ENCR_MASK);
	outw(l_reg,l_data);
}
/*
modify: change '&OERR' to' &0x08'   20150922
*/
unsigned int DrvUART_GetOERR()
{
	unsigned int l_reg,ReturnVal;
	ReturnVal=0;
	l_reg=R_UART_FLAG;
	ReturnVal=((inw(l_reg)&0x08)>>OERR);
	return ReturnVal;
}
/*
modify: change '&ABDOVF' to' &0x01'   20150922
*/
unsigned int DrvUART_GetABDOVF()
{
	unsigned int l_reg,ReturnVal;
	ReturnVal=0;
	l_reg=R_UART_FLAG;
	ReturnVal=((inw(l_reg)&0x01)>>ABDOVF);
	return ReturnVal;
}
/****************************************************************************
*name:DrvUART_CheckTRMT(void)
*function: read transmit shift status bit TRMT;
*input: no
*output: the TRMT bit value of UART(0X40E04);
*version: v1.0
*date:  2013-11-28
*modify:no
****************************************************************************/

unsigned int DrvUART_CheckTRMT(void)
{
	unsigned int l_reg,l_data;
	l_reg=R_UART_FLAG;
	l_data=(inw(l_reg)&0x02)>>TRMT;
	return l_data;

}
/****************************************************************************
*name:DrvUART_ClkEnable();
*function: setting the UART clock source and setting divier;
*input:  uclk: clock source
*              0: external high speed osc
*              1: internal high speed osc
*        uprescale: clock divier;
*output: 
*version: v1.0
*date:  2013-11-28
*modify:no
****************************************************************************/
unsigned int DrvUART_ClkEnable(unsigned int uclk,unsigned int uprescale)
{
	unsigned int Return_value;
	unsigned int Reg_addr,l_data;
	Return_value=0;
	Reg_addr=R_UART_CLOCK;
	if(uprescale<16)
	{
		l_data=inw(Reg_addr)&0xfff0ffff;
		l_data=l_data+((uprescale<<16)&0x000f0000);
		l_data=0xff00ff00+l_data;

	}
	else
	{
		Return_value=1;
		return Return_value;
	}
	if(uclk==0)
	{
		CLRBIT(l_data,USel);
	}
	else
	{
		SETBIT(l_data,USel);
	}
	SETBIT(l_data,UCLK_En);
	outw(Reg_addr,l_data);
	return Return_value;
}
/****************************************************************************
*name     : DrvUART_ClkDisable();
*function : disable UART clock;
*input    : void 
*output   : void
*version  : v1.0
*date     : 2013-11-28
*modify   : no
****************************************************************************/
void DrvUART_ClkDisable(void)
{
	unsigned int l_reg;
	l_reg=R_UART_CLOCK;
	outw(l_reg,0x10000000);
}
/*****************************************************************************
 *name:     DrvUART_Enable()
 *function: enable the UART
 *input:
 *output:
 *version:  v1.0
 *date:     2013-11-13
 *modify:   no
 *****************************************************************************/
void DrvUART_Enable(void)
{
	unsigned int l_reg,l_data;
	l_data=(1<<ENTX_MASK)|(1<<ENSP_MASK)|0xc0;
	l_reg=R_UART_CTRL1;
	outw(l_reg,l_data);
}

/*******************************************************************
*name    : DrvUART_ConfigIO
*function: IOڸΪUARTͨѶڣѡIO
*input   : ioen: 0 رIOڸùܣ1 IOڸù
*          uoutputPin: ѡͨѶIO
*output  : 0SUCCESS ʧ
*Version : v1.0
*date    : 2014-1-17
*modify  : none
*********************************************************************/
unsigned char DrvUART_ConfigIO(unsigned char ioen,unsigned int uOuputPin)
{
	unsigned int l_reg,l_data;
	if((uOuputPin>UAPS_MAX)|(ioen>1)){return E_UART_ERR_OUTPIN;}
	else
	{
		l_data=(uOuputPin<<UAPS)|(UAPS_MAX<<UAPS_MASK)|(ioen<<UAPTOE)|(1<<UAPTOE_MASK);
	    l_reg=R_UART_GPIO;
	    outw(l_reg,l_data);
	    return E_SUCCESS;
	}
}


//**************************************************//
/*ƣinline int putchar(int c)
 * һֽUARTݴ
 *룺c
 *c
 *עNO
 **************************************************/
inline int putchar(int c)
{
///	c = toupper(c);
//****write one byte data to TX buffer and TX***//
#define UART_TX_BUF  0x00040e00+0X14  //UART ݴַ
#define UART_TX_FLAG 0x00040e00+0X04  //UART ״̬ݴַ
	  unsigned int l_data,l_data2;

	  l_data=__nds32__mfsr(NDS32_SR_PSW);
	  l_data2=l_data;
	  if((l_data2&0x01)==1)
	  {
		CLRBIT(l_data,0)
	    __nds32__mtsr(l_data, NDS32_SR_PSW);//disable the GIE
	  }

    outw(UART_BASE+0X14,c);              //write data to TX buffer
    while(inw(UART_BASE+0X04)&0x02);     //check TRMT,wait for TRMT=0;
    while(!(inw(UART_BASE+0X04)&0x02));  //check TRMT,wait for TRMT=1;

	  if((l_data2&0x01)==1)
	  {
	  __nds32__mtsr(l_data2, NDS32_SR_PSW);
	  }

	return c ;
}
//**************************************************//
/*ƣvoid nds_write(const unsigned char *buf, int size)
 * Ͷֽ
 *룺*buf size
 *NO
 *עNO
 **************************************************/
void nds_write(const unsigned char *buf, int size)
{
	int    i;
	for (i = 0; i < size; i++)
		putchar(buf[i]);
}
//**************************************************//
/*ƣvoid uart_init(void)
 *ܣUARTģʼ
 *룺NO
 *NO
 *עNO
 **************************************************/
void uart_init(void)
{
//***************************************************//
//****setup HY16F UART ****//
//**************************************************//
	unsigned int l_reg,l_data,l_temp;

	//***setting UART IO port PT2.2/PT2.3
	if(UART_PIN<4)
	{
		l_data=(1<<(8+UART_TX))+(1<<UART_TX);
		outw(GPIO_BASE+0x0,l_data); //0x0404); //setting PT1 AS OUTPUT
		l_data=(1<<(24+UART_RX))+(1<<(UART_RX+16));
		outw(GPIO_BASE+0x4,l_data); //0x08080000); //setting PT1 AS INPUT
	}
	else if(UART_PIN<=7)
	{
		l_data=(1<<(8+UART_TX))+(1<<UART_TX);
		outw(GPIO_BASE+0x10,l_data); //0x0404); //setting PT2.2 AS OUTPUT
		l_data=(1<<(24+UART_RX))+(1<<(UART_RX+16));
		outw(GPIO_BASE+0x14,l_data); //0x08080000); //setting PT2.3 AS INPUT
	}

//****IOڵĸΪUARTͨѶڹ***//
#define GPIO_COM_CT 0x00040800+0X44  //IOùüĴַ
	l_data=(UART_PIN<<UAPS)|(UAPS_MAX<<UAPS_MASK)|(1<<UAPTOE)|(1<<UAPTOE_MASK);
	outw(GPIO_BASE+0x44,l_data); //0x0b0b); //enable PT2.2/PT2.3 UART IO port function
	//***setting UART clock source ***//
#define MCU_FRE_SUOREC 0x00040300
#define MCU_CLOCK      MCU_FRE_SUOREC+0X08
	if (inw(CLOCK_BASE)&0X00000020)
		outw(CLOCK_BASE+0X08,0x10100000); //setting UART CLOCK SOURCE for OSC_HS
	else
		outw(CLOCK_BASE+0X08,0x30300000); //setting UART CLOCK SOURCE for HAO_CK
	outw(UART_BASE,0xFFc0); //
	outw(UART_BASE+0X08,0xFF08);//


	l_reg=CLOCK_BASE+0X08;
	l_temp=((inw(l_reg)&(UCLK_Div_MAX<<UCLK_Div))>>UCLK_Div);
	l_data=(UART_FRE*1000000/(1<<l_temp)/UART_BR/4);
	                                         //l_reg=R_UART_BRGRH;
	outw(UART_BASE+0X0C,(l_data>>8));        //outw(l_reg,(l_data>>8));
	                                         //l_reg=R_UART_BRGRL;
	outw(UART_BASE+0X10,(l_data&0x000000ff));//outw(l_reg,(l_data&0x000000ff));

}
unsigned int HYprintf_init(
		unsigned int uClock,
		unsigned int uBaudRate ,
		unsigned int uParity,
		unsigned int uDataBits,
		unsigned int uOuputPin)
{
	unsigned int l_reg,l_data,ReturnVal,l_temp;
	ReturnVal=0;
	if(uClock>20){return E_UART_ERR_CLOCK;}
	if(uBaudRate==0){return E_UART_ERR_BAUDRAET;}
	else{
		l_reg=R_UART_CLOCK;
		l_temp=((inw(l_reg)&(UCLK_Div_MAX<<UCLK_Div))>>UCLK_Div);
		l_data=(uClock*1000000/(1<<l_temp)/uBaudRate/4);
		l_reg=R_UART_BRGRH;
		outw(l_reg,(l_data>>8));
		l_reg=R_UART_BRGRL;
		outw(l_reg,(l_data&0x000000ff));
	}
	if (inw(CLOCK_BASE)&0X00000020)
		outw(CLOCK_BASE+0X08,0x10100000); //setting UART CLOCK SOURCE for OSC_HS
	else
		outw(CLOCK_BASE+0X08,0x30300000); //setting UART CLOCK SOURCE for HAO_CK

	if(uParity>2){return E_UART_ERR_PARITY;}
	else{
		switch(uParity)
		{
		    case 0:
		    	l_temp=(0<<ENADD)|(1<<ENADD_MASK);
		    	l_data=(0<<PARITY)|(1<<PARITY_MASK);
		    break;
		    case 1:
		    	l_temp=(1<<ENADD)|(1<<ENADD_MASK);
		    	l_data=(0<<PARITY)|(1<<PARITY_MASK);
		    break;
		    case 2:
		    	l_temp=(1<<ENADD)|(1<<ENADD_MASK);
		    	l_data=(1<<PARITY)|(1<<PARITY_MASK);
		    break;
		    default: break;
		}
		l_reg=R_UART_CTRL2;
		outw(l_reg,l_temp);
		l_reg=R_UART_CTRL1;
		outw(l_reg,l_data);
	}
	if(uDataBits>1){return E_UART_ERR_DATABIT;}
	else{
		if(uDataBits==0){l_data=(0<<TX9)|(1<<TX9_MASK);
		                 l_temp=(0<<RC9)|(1<<RC9_MASK);
		}
		else{l_data=(1<<TX9)|(1<<TX9_MASK);
		     l_temp=(1<<RC9)|(1<<RC9_MASK);
		}
		l_reg=R_UART_CTRL1;
		outw(l_reg,l_data);
		l_reg=R_UART_CTRL2;
		outw(l_reg,l_temp);
	}

	//***setting UART IO port*** //
	if(uOuputPin<4)
	{
		l_data=(1<<(8+uOuputPin*2))+(1<<uOuputPin*2);
		outw(GPIO_BASE+0x0,l_data); //0x0404); //setting PT1 AS OUTPUT
		l_data=(1<<(24+uOuputPin*2+1))+(1<<(uOuputPin*2+1+16));
		outw(GPIO_BASE+0x4,l_data); //0x08080000); //setting PT1 AS INPUT
	}
	else if(uOuputPin<=7)
	{
		l_data=(1<<(8+(uOuputPin-4)*2))+(1<<(uOuputPin-4)*2);
		outw(GPIO_BASE+0x10,l_data); //0x0404); //setting PT2.2 AS OUTPUT
		l_data=(1<<(24+(uOuputPin-4)*2+1))+(1<<((uOuputPin-4)*2+1+16));
		outw(GPIO_BASE+0x14,l_data); //0x08080000); //setting PT2.3 AS INPUT
	}

	if(uOuputPin>UAPS_MAX){return E_UART_ERR_OUTPIN;}
	else{l_data=(uOuputPin<<UAPS)|(UAPS_MAX<<UAPS_MASK)|(1<<UAPTOE)|(1<<UAPTOE_MASK);
	   l_reg=R_UART_GPIO;
	   outw(l_reg,l_data);
	}

	l_data=(1<<ENSP)|(1<<ENSP_MASK)|(1<<ENTX)|(1<<ENTX_MASK);
	l_reg=R_UART_CTRL1;
	outw(l_reg,l_data);
	l_data=(1<<ENCR)|(1<<ENCR_MASK);
	l_reg=R_UART_CTRL2;
	outw(l_reg,l_data);


	return ReturnVal;
}

